@@ -1,9 +1,9 @@
 import concurrent.futures
+import json
 import traceback
 from concurrent.futures import ThreadPoolExecutor
 from typing import Callable, List, Optional
 
-import orjson
 from loguru import logger
 from pydantic import BaseModel, Field
 
@@ -312,14 +312,11 @@ class MultiAgentRouter:
             # Get boss decision using function calling
             boss_response_str = self.function_caller.run(task)
 
-            boss_response_str = orjson.loads(boss_response_str)
+            boss_response_str = json.loads(boss_response_str)
 
             if self.print_on:
                 formatter.print_panel(
-                    # orjson.dumps(boss_response_str, indent=4),
-                    orjson.dumps(
-                        boss_response_str, option=orjson.OPT_INDENT_2
-                    ).decode("utf-8"),
+                    json.dumps(boss_response_str, indent=4),
                     title=self.name,
                 )
 
