@@ -210,7 +210,7 @@ class LiteLLM:
 
     def __init__(
         self,
-        model_name: str = "gpt-4o",
+        model_name: str = "gpt-4.1",
         system_prompt: str = None,
         stream: bool = False,
         temperature: float = 0.5,
@@ -243,7 +243,7 @@ class LiteLLM:
         Initialize the LiteLLM with the given parameters.
 
         Args:
-            model_name (str, optional): The name of the model to use. Defaults to "gpt-4o".
+            model_name (str, optional): The name of the model to use. Defaults to "gpt-4.1".
             system_prompt (str, optional): The system prompt to use. Defaults to None.
             stream (bool, optional): Whether to stream the output. Defaults to False.
             temperature (float, optional): The temperature for the model. Defaults to 0.5.
@@ -411,7 +411,7 @@ class LiteLLM:
     #             response_dict = response.model_dump()
     #         else:
     #             response_dict = response
-                
+
     #         print(f"Response dict: {response_dict}")
 
     #         # Check if tool_calls exists and is not None
@@ -458,7 +458,7 @@ class LiteLLM:
     #             }
     #     except Exception as e:
     #         logger.error(f"Error processing tool calls: {str(e)} Traceback: {traceback.format_exc()}")
-    
+
     def output_for_tools(self, response: any):
         """
         Process and extract tool call information from the LLM response.
@@ -477,25 +477,36 @@ class LiteLLM:
                 for standard tool call responses.
         """
         if self.mcp_call is True:
-            out = response.choices[0].message.tool_calls[0].function
+            tool_calls = response.choices[0].message.tool_calls
 
-            if len(out) > 1:
-                return out
+            # Check if there are multiple tool calls
+            if len(tool_calls) > 1:
+                # Return all tool calls if there are multiple
+                return [
+                    {
+                        "function": {
+                            "name": tool_call.function.name,
+                            "arguments": tool_call.function.arguments,
+                        }
+                    }
+                    for tool_call in tool_calls
+                ]
             else:
-                out = out[0]
-
-            output = {
-                "function": {
-                    "name": out.name,
-                    "arguments": out.arguments,
+                # Single tool call
+                out = tool_calls[0].function
+                output = {
+                    "function": {
+                        "name": out.name,
+                        "arguments": out.arguments,
+                    }
                 }
-            }
-            return output
+                return output
         else:
             out = response.choices[0].message.tool_calls
 
             if isinstance(out, BaseModel):
                 out = out.model_dump()
+
             return out
 
     def output_for_reasoning(self, response: any):
